<?php

class MnglAppHelper
{
  function MnglAppHelper()
  {
  }

  function get_avatar_link($user, $width=0, $height=0, $classes='', $with_link=false)
  {
    if(($width > 0 and $width <= 150) or
       ($height > 0 and $height <= 150))
      $avatar_type = 'thumb';
    else
      $avatar_type = 'full';
    
    if($width > 0 or $height > 0)
      $wrap_style = (($width > 0)?"width: {$width}px;":'') . (($height > 0)?"height: {$height}px;":'');
    else
      $wrap_style = '';

    $avatar_url = $user->get_avatar_url($avatar_type);
    $avatar_dim = $user->get_avatar_dimensions($width);
    
    if($avatar_dim['width'] > 0 or $avatar_dim['height'] > 0)
      $image_style = ' style="' . (($avatar_dim['width'] > 0)?"width: " . $avatar_dim['width'] . "px;":'') . (($avatar_dim['height'] > 0)?"height: " . $avatar_dim['height'] . "px;":'') . '"';
    else
      $image_style = '';

    $avatar_link = '';
    
    if($with_link)
      $avatar_link .= "<a href=\"" . $user->get_profile_url() . "\">";
    
    $avatar_link .= "<img src=\"{$avatar_url}\" class=\"mngl-profile-image {$classes}\"{$image_style}/>";

    if($with_link)
      $avatar_link .= "</a>";
      
    return $avatar_link;
  }
  
  function get_avatar_link_by_id($user_id, $width=0, $height=0, $classes='', $with_link=false)
  {
    $user = MnglUser::get_stored_profile_by_id($user_id);
    return MnglAppHelper::get_avatar_link($user, $width, $height, $classes, $with_link);
  }
  
  function get_avatar_link_by_screenname($screenname, $width=0, $height=0, $classes='', $with_link=false)
  {
    $user = MnglUser::get_stored_profile_by_screenname($screenname);
    return MnglAppHelper::get_avatar_link($user, $width, $height, $classes, $with_link);
  }

  function get_pages()
  {
    global $wpdb;
    
    $query = "SELECT * FROM {$wpdb->prefix}posts WHERE post_status=%s AND post_type=%s";
    
    $query = $wpdb->prepare( $query, "publish", "page" );
    
    $results = $wpdb->get_results( $query );
    
    if($results)
      return $results;
    else
      return array();
  }
  
  // Displays time as in how many seconds / minutes / hours / days / weeks ago
  function time_ago($created_at)
  {
    if($created_at == false)
     return '';

    $ago = time() - strtotime($created_at);
    $ago_str = '';
    
    $minutes = 60;
    $hours   = $minutes * 60;
    $days    = $hours * 24;
    $weeks   = $days * 7;
    $months  = $days * 30;
    $years   = $days * 365;
    
    if($ago < $minutes)
    {
      $ago_str = sprintf(__ngettext("%d second ago","%d seconds ago", $ago, 'mingle'), $ago);
    }
    else if($ago < $hours)
    {
      $ago_minutes = (int)($ago / $minutes);
      
      $ago_str = sprintf(__ngettext("%d minute ago", "%d minutes ago", $ago_minutes, 'mingle'), $ago_minutes);
    }
    else if($ago < $days)
    {
      $ago_hours = (int)($ago / $hours);
      
      $ago_str = sprintf(__ngettext("%d hour ago", "%d hours ago", $ago_hours, 'mingle'), $ago_hours);
    }
    else if($ago < $weeks)
    {
      $ago_days = (int)($ago / $days);
      
      $ago_str = sprintf(__ngettext("yesterday", "%d days ago", $ago_days, 'mingle'), $ago_days);
    }
    else if($ago < $months)
    {
      $ago_weeks = (int)($ago / $weeks);
      
      $ago_str = sprintf(__ngettext("last week", "%d weeks ago", $ago_weeks, 'mingle'), $ago_weeks);
    }
    else if($ago < $years)
    {
      $ago_months = (int)($ago / $months);
      
      $ago_str = sprintf(__ngettext("last month", "%d months ago", $ago_months, 'mingle'), $ago_months);
    }
    else
    {
      $ago_years = (int)($ago / $years);
      
      $ago_str = sprintf(__ngettext("last year", "%d years ago", $ago_years, 'mingle'), $ago_years);
    }
    
    return $ago_str;
  }
  
  function value_is_selected($field_name, $field_value, $selected_value)
  {
    if( (isset($_POST[$field_name]) and $_POST[$field_name] == $selected_value) or
        (!isset($_POST[$field_name]) and $field_value == $selected_value) )
      echo ' selected="selected"';
  }
  
  function value_is_checked($field_name, $field_value)
  {
    if( (isset($_POST) and $_POST[$field_name] == 'on') or
        (!isset($_POST) and $field_value == 'on') )
      echo ' checked="checked"';
  }

  function value_is_checked_with_array($field_name, $index, $field_value)
  {
    if( ( $_POST['action'] == 'process_form' and isset( $_POST[ $field_name ][ $index ] ) ) or
        ( $_POST['action'] != 'process_form' and isset($field_value) ) )
      echo ' checked="checked"';
  }
  
  function powered_by()
  {
    $show_powered_by = apply_filters('mngl-show-powered-by',true);
    
    if($show_powered_by)
    {
    ?>
      <p style="font-size: 8px;"><?php printf(__('Powered by %1$sMingle%2$s', 'mingle'), '<span><img src="' . MNGL_URL . '/images/mingle_16.png" alt="Small Mingle Icon" width="10px" height="10px">&nbsp;<a href="http://blairwilliams.com/mingle" style="text-decoration: none;">', '</a></span>'); ?></p>
    <?php
    }
  }
  
  function get_extension( $mimetype )
  {
    switch( $mimetype )
    {
      case "application/msword":
      case "application/rtf":
      case "text/richtext":
        return "doc";
      case "application/vnd.ms-excel":
        return "xls";
      case "application/vnd.ms-powerpoint":
        return "ppt";
      case "application/pdf":
        return "pdf";
      case "application/zip":
        return "zip";
      case "image/jpeg":
        return "jpg";
      case "image/gif":
        return "gif";
      case "image/png":
        return "png";
      case "image/tiff":
        return "tif";
      case "text/plain":
        return "txt";
      case "text/html":
        return "html";
      case "video/quicktime":
        return "mov";
      case "video/x-msvideo":
        return "avi";
      case "video/x-ms-wmv":
        return "wmv";
      case "video/ms-wmv":
        return "wmv";
      case "video/mpeg":
        return "mpg";
      case "audio/mpg":
        return "mp3";
      case "audio/x-m4a":
        return "aac";
      case "audio/m4a":
        return "aac";
      case "audio/x-wav":
        return "wav";
      case "audio/wav":
        return "wav";
      case "application/x-zip-compressed":
        return "zip";
      default:
        return "bin";
    }
  }
  
  
  function decode_unicode($val)
  { 
    $val = preg_replace_callback("/%u([0-9a-fA-F]{4})/",
                                 create_function(
                                   '$matches',
                                   'return html_entity_decode("&#".hexdec($matches[1]).";",ENT_COMPAT,"UTF-8");'
                                 ),
                                 $val);
    return $val;
  }

}

?>
