<?php

class MnglUser
{ 
  var $profile_name;
  
  var $id; // ID from the current user record
  var $avatar;
  var $avatar_thumb;
  var $avatar_meta;
  var $avatar_thumb_meta;
  var $db_first_name; // For database use only
  var $db_last_name; // For database use only
  var $screenname;
  var $email;
  var $url;
  var $location;
  var $sex;
  var $bio;
  var $fields;
  var $hashed_password;
  
  var $id_str;
  var $avatar_str;
  var $first_name_str;
  var $last_name_str;
  var $screenname_str;
  var $email_str;
  var $url_str;
  var $location_str;
  var $sex_str;
  var $bio_str;
  var $fields_str;
  var $password_str;
  var $password_confirm_str;
  
  var $full_name;
  var $first_name;
  var $last_name;
  var $his_her; // calculated from $sex var
  var $him_her; // calculated from $sex var
  var $he_she; // calculated from $sex var

  // Notification Settings
  var $hide_notifications;
  
  var $hide_notifications_str;
  
  // Privacy Settings
  var $privacy;
  
  var $privacy_str;

  function MnglUser( $id = '')
  {
    $this->set_default_options( $id );
  }

  function set_default_options( $id = '' )
  {
    global $mngl_options, $current_user;
    get_currentuserinfo();
    
    if( empty($id) )
      $user = $current_user;
    else
      $user = get_userdata($id);
    
    // We're abstracting and simplifying user objects here
    $this->id         = $user->ID;
    $this->screenname = $user->user_login;
    $this->email      = $user->user_email;
    $this->url        = $user->user_url;

    // Set string keys
    $this->profile_name           = 'mngl_user_profile';
    $this->id_str                 = 'mngl_user_id';
    $this->avatar_str             = 'mngl_user_avatar';
    $this->first_name_str         = 'mngl_user_first-name';
    $this->last_name_str          = 'mngl_user_last-name';
    $this->screenname_str         = 'mngl_user_screenname';
    $this->email_str              = 'mngl_user_email';
    $this->url_str                = 'mngl_user_url';
    $this->location_str           = 'mngl_user_location';
    $this->sex_str                = 'mngl_user_sex';
    $this->bio_str                = 'mngl_user_bio';
    $this->fields_str             = 'mngl_user_fields';
    $this->hide_notifications_str = 'mngl_user_hide_notifications';
    $this->privacy_str            = 'mngl_user_privacy';
    $this->password_str           = 'mngl_password';
    $this->password_confirm_str   = 'mngl_password_confirm';
    
    // Get some metadata aggregated dogg!
    $this->db_first_name      = get_usermeta($user->ID, 'first_name' );
    $this->db_last_name       = get_usermeta($user->ID, 'last_name' );
    $this->avatar             = get_usermeta($user->ID, 'mngl_avatar' );
    $this->avatar_meta        = get_usermeta($user->ID, 'mngl_avatar_meta' );
    $this->avatar_thumb       = get_usermeta($user->ID, 'mngl_avatar_thumb' );
    $this->avatar_meta_thumb  = get_usermeta($user->ID, 'mngl_avatar_meta_thumb' );
    $this->location           = get_usermeta($user->ID, 'mngl_location' );
    $this->sex                = get_usermeta($user->ID, 'mngl_sex' );
    $this->bio                = get_usermeta($user->ID, 'description' );
    $this->fields             = get_usermeta($user->ID, 'mngl_fields' );
    $this->hide_notifications = get_usermeta($user->ID, $this->hide_notifications_str );
    $this->privacy            = get_usermeta($user->ID, $this->privacy_str );

    // Set Defaults
    if(!isset($this->hide_notifications) or !is_array($this->hide_notifications))
      $this->hide_notifications = array();

    if(!isset($this->privacy) or !$this->privacy)
      $this->privacy = 'private';

    if(!isset($this->avatar) or !$this->avatar)
      $this->avatar = '';
    else
    {
      $avmeta = $this->get_avatar_meta();

      if( (int)$avmeta[0] > 700 or
          (int)$avmeta[1] > 700 )
      {
        require_once(ABSPATH.'wp-admin/includes/image.php');
        require_once(ABSPATH.'wp-includes/media.php');
        $resized_image = image_resize( ABSPATH . $this->avatar, 700, 700, false, null, dirname( ABSPATH . $this->avatar ) );

        if ( !is_wp_error($resized_image) and $resized_image )
        {
          $this->avatar = str_replace(ABSPATH,'',$resized_image);
          $this->avatar_meta = getimagesize(ABSPATH . $this->avatar);
          update_usermeta( $this->id, 'mngl_avatar', $this->avatar );
          update_usermeta( $this->id, 'mngl_avatar_meta', $this->avatar_meta );
        }
      }
    }

    if(!isset($this->avatar_thumb) or !$this->avatar_thumb)
    {
      if(!isset($this->avatar) or !$this->avatar or empty($this->avatar))
        $this->avatar_thumb = $this->avatar;
      else
      {
        $this->resize_and_crop_thumb();
        update_usermeta( $this->id, 'mngl_avatar_thumb', $this->avatar_thumb );
        update_usermeta( $this->id, 'mngl_avatar_meta_thumb', $this->avatar_meta_thumb );
      }
    }
    else
    {
      $avmeta = $this->get_avatar_meta('thumb');

      if( (int)$avmeta[0] != (int)$avmeta[1] )
      {
        $this->resize_and_crop_thumb();
        update_usermeta( $this->id, 'mngl_avatar_thumb', $this->avatar_thumb );
        update_usermeta( $this->id, 'mngl_avatar_meta_thumb', $this->avatar_meta_thumb );
      }
    }

    if(!isset($this->sex) or !$this->sex)
      $this->sex = '';

    if(!isset($this->hashed_password) or !$this->hashed_password)
      $this->hashed_password = '';

    if(!isset($this->fields) or !$this->fields)
      $this->fields = array();
    
    // Calculate meta data settings
    if( $mngl_options->display_name_type == 'fullname' and 
        !empty( $this->db_first_name ) and 
        !empty( $this->db_last_name ) )
    {
      $this->full_name  = "{$this->db_first_name} {$this->db_last_name}";
      $this->first_name = $this->db_first_name;
      $this->last_name  = $this->db_last_name;
    }
    else
    {
      $this->full_name  = $this->screenname;
      $this->first_name = $this->screenname;
      $this->last_name  = $this->screenname;
    }
    
    // Calculate pronouns to use depending on gender
    if($this->sex == 'female')
    {
      $this->his_her = __('Her', 'mingle');
      $this->him_her = __('Her', 'mingle');
      $this->he_she  = __('She', 'mingle');
    }
    else if($this->sex == 'male')
    {
      $this->his_her = __('His', 'mingle');
      $this->him_her = __('Him', 'mingle');
      $this->he_she  = __('He', 'mingle');
    }
    else
    {
      $this->his_her = __('His/Her', 'mingle');
      $this->him_her = __('Him/Her', 'mingle');
      $this->he_she  = __('He/She', 'mingle');
    }
  }
  
  function validate($params,$errors)
  {   
    if( isset( $params[ $this->password_str ] ) and
        !empty( $params[ $this->password_str ]) and
        ( $params[ $this->password_str ] != $params[ $this->password_confirm_str ] ) )
    {
      $errors[] = __("Password Must Match Password Confirmation.", 'mingle');
    }
    
    return $errors;
  }
  
  function update($params)
  {
    global $mngl_options;
    
    // Only the current user can modify his/her own profile
    if($this->is_logged_in_and_current_user($this->id))
    { 
      $this->id                 = (int)$params[ $this->id_str ];
      $this->db_first_name      = $params[ $this->first_name_str ];
      $this->db_last_name       = $params[ $this->last_name_str ];
      $this->screenname         = $params[ $this->screenname_str ];
      $this->email              = $params[ $this->email_str ];
      $this->url                = $params[ $this->url_str ];
      $this->location           = $params[ $this->location_str ];
      $this->sex                = $params[ $this->sex_str ];
      $this->bio                = $params[ $this->bio_str ];
      $this->fields             = $params[ $this->fields_str ];
      $this->hide_notifications = $params[ $this->hide_notifications_str ];
      $this->privacy            = $params[ $this->privacy_str ];
      
      if( isset( $params[ $this->password_str ] ) and
          !empty( $params[ $this->password_str ]) and
          ( $params[ $this->password_str ] == $params[ $this->password_confirm_str ] ) )
      {
        $this->hashed_password = wp_hash_password( $params[ $this->password_str ] );
      }
      
      // Upload the avatar 
      if( isset($_FILES[ $this->avatar_str ]) and 
          !empty($_FILES[ $this->avatar_str ]['name']) and
          (int)$_FILES[ $this->avatar_str ]['size'] > 0 )
      {
        $target_path = get_option('upload_path');
        if(!file_exists($target_path))
          @mkdir($target_path."/");
      
        $target_path = $target_path . "/mingle";
        if(!file_exists($target_path))
          @mkdir($target_path."/");
    
        $target_path = $target_path . "/avatars";
        if(!file_exists($target_path))
          @mkdir($target_path."/");
        
        // Using WordPress' built in resize capabilies using GD
        require_once(ABSPATH.'wp-admin/includes/image.php');
        require_once(ABSPATH.'wp-includes/media.php');
        $tmp_image = $target_path . '/tmp_' . md5(trim(strtolower($this->email)));
        move_uploaded_file($_FILES[ $this->avatar_str ]['tmp_name'], $tmp_image);
        $resized_image = image_resize( $tmp_image, 700, 700, false, null, $target_path );
      
        if ( !is_wp_error($resized_image) and $resized_image )
        {
          $full_image = $resized_image;
          unlink($tmp_image);
        }
        else
          $full_image = $tmp_image;
        
        $this->avatar_meta = getimagesize($full_image);
        $ext = MnglAppHelper::get_extension($this->avatar_meta['mime']); 
        $image_path = $target_path . '/' . md5(trim(strtolower($this->email))) . "_" . time() . ".{$ext}";

        // Rename the full image
        copy($full_image, $image_path);
        unlink($full_image);

        // Destroy old avatar if it exists
        if(!empty($this->avatar) and !$this->avatar and file_exists(ABSPATH . $this->avatar))
          unlink(ABSPATH . $this->avatar);

        $this->avatar = str_replace(ABSPATH,'',$image_path);

        $this->resize_and_crop_thumb();
      }
    }
  }
  
  function store()
  {
    global $wpdb;
    
    // Only the current user can store his/her own profile
    if($this->is_logged_in_and_current_user($this->id))
    {
      // Update User Table Fields
      if( !empty( $this->hashed_password ) )
      {
        $query = "UPDATE {$wpdb->prefix}users SET user_login=%s, user_email=%s, user_url=%s, user_pass=%s WHERE id=%d";
        $query = $wpdb->prepare($query, $this->screenname, $this->email, $this->url, $this->hashed_password, $this->id);
      }
      else
      {
        $query = "UPDATE {$wpdb->prefix}users SET user_login=%s, user_email=%s, user_url=%s WHERE id=%d";
        $query = $wpdb->prepare($query, $this->screenname, $this->email, $this->url, $this->id);
      }
      $wpdb->query($query);
      
      // Update User Meta Fields
      update_usermeta( $this->id, 'first_name',             $this->db_first_name );
      update_usermeta( $this->id, 'last_name',              $this->db_last_name );
      update_usermeta( $this->id, 'mngl_avatar',            $this->avatar );
      update_usermeta( $this->id, 'mngl_avatar_meta',       $this->avatar_meta );
      update_usermeta( $this->id, 'mngl_avatar_thumb',      $this->avatar_thumb );
      update_usermeta( $this->id, 'mngl_avatar_thumb_meta', $this->avatar_thumb_meta );
      update_usermeta( $this->id, 'mngl_location',          $this->location );
      update_usermeta( $this->id, 'mngl_sex',               $this->sex );
      update_usermeta( $this->id, 'description',            $this->bio );
      update_usermeta( $this->id, 'mngl_fields',            $this->fields );
      
      update_usermeta( $this->id, $this->hide_notifications_str, $this->hide_notifications );
      update_usermeta( $this->id, $this->privacy_str,            $this->privacy );
    }
  }
  
  function get_all( $where='', $order_by='', $limit='' )
  {
    global $wpdb, $mngl_friend;
    
    $where    = ((empty($where))?'':" WHERE {$where}");
    $order_by = ((empty($order_by))?'':" ORDER BY {$order_by}");
    $limit    = ((empty($limit))?'':" LIMIT {$limit}");
    
    $query_str = "SELECT * FROM {$wpdb->prefix}users{$where}{$order_by}{$limit}";
    $query = $wpdb->prepare( $query_str, $user_id );

    return $wpdb->get_results($query);
  }
  
  function get_user_array( $offset=0, $limit=50, $order_by='' )
  {
    global $wpdb;

    if( MnglUser::get_count() <= 0 )
      return false;

    $users = MnglUser::get_all( '', $order_by, "{$offset},{$limit}" );

    $users_a = array();
    
    foreach ($users as $user) 
      $users_a[] = MnglUser::get_stored_profile_by_id($user->ID);
    
    return $users_a;
  }
  
  function get_avatar_url($type='full')
  {
    global $mngl_blogurl;
    
    if($type=='full')
    {
      $avfile = $this->avatar;
      $avmeta = $this->avatar_meta;
    }
    else if($type=='thumb')
    {
      $avfile = $this->avatar_thumb;
      $avmeta = $this->avatar_thumb_meta;
    }
    
    if(empty($avfile) or !file_exists( ABSPATH . $avfile ))
    {  
      $mystery_url = MNGL_URL . "/images/mystery_avatar.png";
      if(!empty($this->email))
      {
        $email_hash = md5(trim(strtolower($this->email)));
        $size = (($type=="thumb")?'150':'300');
        return "http://www.gravatar.com/avatar/{$email_hash}.jpg?s={$size}&d=".urlencode($mystery_url);
      }
      else
        return $mystery_url;
    }
    else
      return "{$mngl_blogurl}/{$avfile}";
  }
  
  function get_avatar_meta($type="full")
  {  
    if($type=='full')
    {
      $avfile = &$this->avatar;
      $avmeta = &$this->avatar_meta;
    }
    else if($type=='thumb')
    {
      $avfile = &$this->avatar_thumb;
      $avmeta = &$this->avatar_thumb_meta;
    }

    if(!isset($avmeta) or empty($avmeta))
    {
      if(!empty($avfile) and file_exists( ABSPATH . $avfile ))
      {
        $avmeta = getimagesize( ABSPATH . $avfile );
        $this->store();
      }
    }

    return $avmeta;
  }
  
  function get_avatar_mime_type($type='full')
  {
    $avmeta = $this->get_avatar_meta($type);
    if(!empty($avmeta))
      return $avmeta['mime'];
    else
      return '';
  }
  
  function resize_and_crop_thumb()
  {
    // resize if there isn't a thumb and there is an image
    // Using WordPress' built in resize capabilies using GD
    require_once(ABSPATH.'wp-admin/includes/image.php');
    require_once(ABSPATH.'wp-includes/media.php');
    $thumb_resized_image = image_resize( (ABSPATH . $this->avatar), 150, 150, true, null, dirname( ABSPATH . $this->avatar ) );
    
    if ( !is_wp_error($thumb_resized_image) and 
         $thumb_resized_image and 
         $this->avatar_thumb_meta = getimagesize($thumb_resized_image) )
      $this->avatar_thumb = str_replace( ABSPATH, '', $thumb_resized_image );
    else
    {
      $this->avatar_thumb = $this->avatar;
      $this->avatar_thumb_meta = $this->avatar_meta;
    }
  }
  
  function get_avatar_dimensions($constrain_width=0,$constrain_height=0)
  {
    if(($constrain_width > 0 and $constrain_width <= 150) or
       ($constrain_height > 0 and $constrain_height <= 150))
      $avatar_type = 'thumb';
    else
      $avatar_type = 'full';

    $avmeta = $this->get_avatar_meta($avatar_type);

    if(!empty($avmeta))
    {
      $real_width  = $avmeta[0];
      $real_height = $avmeta[1];

      if($constrain_width==0 and $constrain_height==0)
        return array( "width" => $real_width, "height" => $real_height );
      else if($constrain_width==0 and $constrain_height > 0)
        return array( "width" => (int)(( (float)$constrain_height / (float)$real_height ) * (float)$real_width ), "height" => $constrain_height );
      else if($constrain_width > 0 and $constrain_height == 0)
        return array( "width" => $constrain_width, "height" => (int)(( (float)$constrain_width / (float)$real_width ) * (float)$real_height ) );
      else
        return array( "width" => $constrain_width, "height" => $constrain_height );
    }
    else
    {
      if( $constrain_width==0 and $constrain_height==0 )
        return array( "width" => 1, "height" => 1 );
      else if( $constrain_width==0 and $constrain_height > 0 )
        return array( "width" => $constrain_height, "height" => $constrain_height );
      else if( $constrain_width > 0 and $constrain_height == 0 )
        return array( "width" => $constrain_width, "height" => $constrain_width );
      else
        return array( "width" => $constrain_width, "height" => $constrain_height );
    }
  }
  
  function get_profile_url()
  {
    global $mngl_options;
    
    if(isset($mngl_options->profile_page_id) and $mngl_options->profile_page_id != 0)
    {
      $permalink = get_permalink($mngl_options->profile_page_id);
      $param_char = ((preg_match("#\?#",$permalink))?'&':'?');
      return "{$permalink}{$param_char}u={$this->screenname}";
    }
    
    return '';
  }
  
  function get_friends_url()
  {
    global $mngl_options;

    if(isset($mngl_options->friends_page_id) and $mngl_options->friends_page_id != 0)
    {
      $permalink = get_permalink($mngl_options->friends_page_id);
      $param_char = ((preg_match("#\?#",$permalink))?'&':'?');
      return "{$permalink}{$param_char}u={$this->screenname}";
    }
    
    return '';
  }
  
  function update_status($status)
  {  
    if(!empty($status))
    {
      update_usermeta( $this->id, 'mngl-user-status',  $status );
      update_usermeta( $this->id, 'mngl-user-status-time', date('c') );
    }
  }
  
  function get_status()
  {
    return get_usermeta( $this->id, 'mngl-user-status' );
  }
  
  function get_status_time()
  {
    return get_usermeta( $this->id, 'mngl-user-status-time' );
  }
  
  function get_stored_profile_by_id($user_id = '')
  {
    $user = '';

    if(!empty($user_id))
      $user = get_userdata($user_id);
    
    return MnglUser::get_stored_profile($user);
  }
  
  function get_stored_profile_by_screenname($screenname = '')
  {
    $user = '';
    
    if(!empty($screenname))
      $user = get_userdatabylogin($screenname);
    
    return MnglUser::get_stored_profile($user);
  }
  
  function get_stored_profile($user = '')
  {
    if(empty($user))
    {
      global $current_user;
      get_currentuserinfo();
      $user = $current_user;
    }
    
    return new MnglUser($user->ID);
  }
  
  function get_stored_profiles($offset=0, $limit=50)
  {
    global $wpdb;

    $limit_str = (($limit > 0)?" LIMIT {$offset},{$limit}":'');
    $query = "SELECT ID FROM {$wpdb->prefix}users ORDER BY user_login ASC{$limit_str}";
    $user_ids = $wpdb->get_col($query,0);
    $profiles = array();
    
    foreach ($user_ids as $user_id)
      $profiles[] = MnglUser::get_stored_profile_by_id($user_id);
    
    return $profiles;
  }
  
  function get_count()
  {
    global $wpdb;

    $query = "SELECT COUNT(*) FROM {$wpdb->prefix}users";
    
    return $wpdb->get_var($query);
  }
  
  function is_logged_in_and_current_user($user_id)
  {
    global $current_user;
    get_currentuserinfo();

    return (is_user_logged_in() and ($current_user->ID == $user_id));
  }
  
  function is_logged_in_and_an_admin()
  {
    return (is_user_logged_in() and current_user_can('level_10'));
  }
}
?>
